import datetime
from .const import CONF_EVENT as CONF_EVENT, CalendarEntityFeature as CalendarEntityFeature, EVENT_DESCRIPTION as EVENT_DESCRIPTION, EVENT_END as EVENT_END, EVENT_RECURRENCE_ID as EVENT_RECURRENCE_ID, EVENT_RECURRENCE_RANGE as EVENT_RECURRENCE_RANGE, EVENT_RRULE as EVENT_RRULE, EVENT_START as EVENT_START, EVENT_SUMMARY as EVENT_SUMMARY, EVENT_UID as EVENT_UID
from _typeshed import Incomplete
from aiohttp import web
from collections.abc import Callable as Callable, Iterable
from homeassistant.components import frontend as frontend, http as http, websocket_api as websocket_api
from homeassistant.components.websocket_api import ERR_NOT_FOUND as ERR_NOT_FOUND, ERR_NOT_SUPPORTED as ERR_NOT_SUPPORTED
from homeassistant.components.websocket_api.connection import ActiveConnection as ActiveConnection
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import STATE_OFF as STATE_OFF, STATE_ON as STATE_ON
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE, time_period_str as time_period_str
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.template import DATE_STR_FORMAT as DATE_STR_FORMAT
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.util import dt as dt
from typing import Any

_LOGGER: Incomplete
DOMAIN: str
ENTITY_ID_FORMAT: Incomplete
SCAN_INTERVAL: Incomplete
VALID_FREQS: Incomplete

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
def get_date(date: dict[str, Any]) -> datetime.datetime: ...

class CalendarEvent:
    start: Union[datetime.date, datetime.datetime]
    end: Union[datetime.date, datetime.datetime]
    summary: str
    description: Union[str, None]
    location: Union[str, None]
    uid: Union[str, None]
    recurrence_id: Union[str, None]
    rrule: Union[str, None]
    @property
    def start_datetime_local(self) -> datetime.datetime: ...
    @property
    def end_datetime_local(self) -> datetime.datetime: ...
    @property
    def all_day(self) -> bool: ...
    def as_dict(self) -> dict[str, Any]: ...
    def __init__(self, start, end, summary, description, location, uid, recurrence_id, rrule) -> None: ...

def _event_dict_factory(obj: Iterable[tuple[str, Any]]) -> dict[str, str]: ...
def _api_event_dict_factory(obj: Iterable[tuple[str, Any]]) -> dict[str, Any]: ...
def _get_datetime_local(dt_or_d: Union[datetime.datetime, datetime.date]) -> datetime.datetime: ...
def _get_api_date(dt_or_d: Union[datetime.datetime, datetime.date]) -> dict[str, str]: ...
def extract_offset(summary: str, offset_prefix: str) -> tuple[str, datetime.timedelta]: ...
def is_offset_reached(start: datetime.datetime, offset_time: datetime.timedelta) -> bool: ...
def _validate_rrule(value: Any) -> str: ...

class CalendarEntity(Entity):
    @property
    def event(self) -> Union[CalendarEvent, None]: ...
    @property
    def state_attributes(self) -> Union[dict[str, Any], None]: ...
    @property
    def state(self) -> str: ...
    async def async_get_events(self, hass: HomeAssistant, start_date: datetime.datetime, end_date: datetime.datetime) -> list[CalendarEvent]: ...
    async def async_create_event(self, **kwargs: Any) -> None: ...
    async def async_delete_event(self, uid: str, recurrence_id: Union[str, None] = ..., recurrence_range: Union[str, None] = ...) -> None: ...
    async def async_update_event(self, uid: str, event: dict[str, Any], recurrence_id: Union[str, None] = ..., recurrence_range: Union[str, None] = ...) -> None: ...

class CalendarEventView(http.HomeAssistantView):
    url: str
    name: str
    component: Incomplete
    def __init__(self, component: EntityComponent[CalendarEntity]) -> None: ...
    async def get(self, request: web.Request, entity_id: str) -> web.Response: ...

class CalendarListView(http.HomeAssistantView):
    url: str
    name: str
    component: Incomplete
    def __init__(self, component: EntityComponent[CalendarEntity]) -> None: ...
    async def get(self, request: web.Request) -> web.Response: ...

def _has_same_type(*keys: Any) -> Callable[[dict[str, Any]], dict[str, Any]]: ...
def _has_consistent_timezone(*keys: Any) -> Callable[[dict[str, Any]], dict[str, Any]]: ...
def _is_sorted(*keys: Any) -> Callable[[dict[str, Any]], dict[str, Any]]: ...
async def handle_calendar_event_create(hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]) -> None: ...
async def handle_calendar_event_delete(hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]) -> None: ...
async def handle_calendar_event_update(hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]) -> None: ...
